@@ -1,5 +1,5 @@
 ###
-# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
+# Copyright 2016-2023 Hewlett Packard Enterprise, Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,34 +17,47 @@
 # -*- coding: utf-8 -*-
 """ Load Command for RDMC """
 
-import os
-import sys
 import json
+import os
 import shlex
 import subprocess
-
+import sys
 from datetime import datetime
-from argparse import ArgumentParser, SUPPRESS
 
 from six.moves import queue
 
 import redfish.ris
-
 from redfish.ris.rmc_helper import LoadSkipSettingError
-from rdmc_helper import (
-    ReturnCodes,
-    InvalidCommandLineError,
-    InvalidCommandLineErrorOPTS,
-    InvalidFileFormattingError,
-    NoChangesFoundOrMadeError,
-    InvalidFileInputError,
-    NoDifferencesFoundError,
-    MultipleServerConfigError,
-    InvalidMSCfileInputError,
-    Encryption,
-)
 
-from rdmc_base_classes import HARDCODEDLIST
+try:
+    from rdmc_helper import (
+        Encryption,
+        InvalidCommandLineError,
+        InvalidCommandLineErrorOPTS,
+        InvalidFileFormattingError,
+        InvalidFileInputError,
+        InvalidMSCfileInputError,
+        MultipleServerConfigError,
+        NoChangesFoundOrMadeError,
+        ReturnCodes,
+    )
+except ImportError:
+    from ilorest.rdmc_helper import (
+        Encryption,
+        InvalidCommandLineError,
+        InvalidCommandLineErrorOPTS,
+        InvalidFileFormattingError,
+        InvalidFileInputError,
+        InvalidMSCfileInputError,
+        MultipleServerConfigError,
+        NoChangesFoundOrMadeError,
+        ReturnCodes,
+    )
+
+try:
+    from rdmc_helper import HARDCODEDLIST
+except:
+    from ilorest.rdmc_helper import HARDCODEDLIST
 
 # default file name
 __filename__ = "ilorest.json"
@@ -69,7 +82,7 @@ class LoadCommand:
             "hostname> -u admin -p password",
             "summary": "Loads the server configuration settings from a file.",
             "aliases": [],
-            "auxcommands": ["CommitCommand", "SelectCommand"],
+            "auxcommands": ["CommitCommand", "SelectCommand", "RawPatchCommand"],
         }
         self.filenames = None
         self.mpfilename = None
@@ -78,6 +91,36 @@ class LoadCommand:
         self.rdmc = None
         self.auxcommands = dict()
 
+    def securebootremovereadonly(self, tmp):
+        templist = [
+            "SecureBootCurrentBoot",
+            "@odata.etag",
+            "Id",
+            "AutoNeg",
+            "FQDN",
+            "FullDuplex",
+            "HostName",
+            "IPv6Addresses",
+            "IPv6DefaultGateway",
+            "LinkStatus",
+            "MaxIPv6StaticAddresses",
+            "Name",
+            "NameServers",
+            "PermanentMACAddress",
+            "ConfigurationSettings",
+            "InterfaceType",
+            "NICSupportsIPv6",
+            "DomainName",
+            "IPV6",
+            "DNSServers",
+            "MACAddress",
+            "VLAN",
+            "SpeedMbps",
+            "DateTime",
+        ]
+        remove_data = self.rdmc.app.removereadonlyprops(tmp, False, True, templist)
+        return remove_data
+
     def run(self, line, help_disp=False):
         """Main load worker function
 
@@ -106,8 +149,7 @@ class LoadCommand:
         for files in self.filenames:
             if not os.path.isfile(files):
                 raise InvalidFileInputError(
-                    "File '%s' doesn't exist. Please "
-                    "create file by running save command." % files
+                    "File '%s' doesn't exist. Please create file by running save command." % files
                 )
             if options.encryption:
                 with open(files, "rb") as myfile:
@@ -118,26 +160,8 @@ class LoadCommand:
                 try:
                     with open(files, "r") as myfile:
                         loadcontents = json.load(myfile)
-                #        for loadcontent in loadcontents:
-                #            if 'Comments' in loadcontent:
-                #                if "SerialNumber" in loadcontent["Comments"]:
-                #                    del (loadcontent["Comments"]["SerialNumber"])
-                #            for _, v in loadcontent.items():
-                #                for _, v in v.items():
-                #                    if "Attributes" in v:
-                #                        if 'SerialNumber' in v['Attributes']:
-                #                            del (v["Attributes"]["SerialNumber"])
-                #                        if 'ProductId' in v['Attributes']:
-                #                            del (v["Attributes"]["ProductId"])
-                #                    else:
-                #                        if 'SerialNumber' in v:
-                #                            del (v["SerialNumber"])
-                #                        if 'ProductId' in v:
-                #                            del (v["ProductId"])
                 except:
-                    raise InvalidFileFormattingError(
-                        "Invalid file formatting " "found in file %s" % files
-                    )
+                    raise InvalidFileFormattingError("Invalid file formatting found in file %s" % files)
 
             if options.mpfilename:
                 mfile = options.mpfilename
@@ -149,11 +173,10 @@ class LoadCommand:
                 if self.runmpfunc(mpfile=mfile, lfile=files, outputdir=outputdir):
                     return ReturnCodes.SUCCESS
                 else:
-                    raise MultipleServerConfigError(
-                        "One or more servers " "failed to load given configuration."
-                    )
+                    raise MultipleServerConfigError("One or more servers failed to load given configuration.")
 
             results = False
+            eth_conf = False
             validation_errs = []
 
             for loadcontent in loadcontents:
@@ -163,6 +186,41 @@ class LoadCommand:
                     if content == "Comments":
                         continue
 
+                    if "EthernetInterface" in content:
+                        if options.force_network_config:
+                            import platform
+                            import tempfile
+
+                            patchpath = "/redfish/v1/Managers/1/EthernetInterfaces/1/"
+
+                            try:
+                                payload = loadcontent[content][patchpath]
+                            except KeyError:
+                                continue
+
+                            payload = loadcontent[content][patchpath]
+                            remove_odata = self.securebootremovereadonly(payload)
+                            payload.update(remove_odata)
+                            if "StaticNameServers" in payload:
+                                payload["StaticNameServers"] = [
+                                    item for item in payload["StaticNameServers"] if item != "::"
+                                ]
+                            tempdir = "/tmp" if platform.system() == "Darwin" else tempfile.gettempdir()
+                            temp_file = os.path.join(tempdir, "temp_patch.json")
+                            out_file = open(temp_file, "w")
+                            patch_payload = {patchpath: payload}
+                            json.dump(patch_payload, out_file, indent=6)
+                            out_file.close()
+                            self.auxcommands["rawpatch"].run(temp_file + " --service")
+                            os.remove(temp_file)
+                            eth_conf = True
+                            continue
+                        else:
+                            self.rdmc.ui.printer(
+                                "Skipping network configurations as" " --force_network_config is not included.\n"
+                            )
+                            continue
+
                     inputlist.append(content)
                     if options.biospassword:
                         inputlist.extend(["--biospassword", options.biospassword])
@@ -173,6 +231,8 @@ class LoadCommand:
 
                     try:
                         for _, items in loaddict.items():
+                            remove_odata = self.securebootremovereadonly(items)
+                            items.update(remove_odata)
                             try:
                                 if self.rdmc.app.loadset(
                                     seldict=items,
@@ -180,7 +240,7 @@ class LoadCommand:
                                     uniqueoverride=options.uniqueoverride,
                                 ):
                                     results = True
-                            except LoadSkipSettingError as excp:
+                            except LoadSkipSettingError:
                                 returnvalue = True
                                 results = True
                             except:
@@ -194,6 +254,13 @@ class LoadCommand:
             try:
                 if results:
                     self.auxcommands["commit"].commitfunction(options=options)
+                    reboot_needed = False
+                    for n in inputlist:
+                        n = n.lower()
+                        if "bios" in n or "boot" in n or "iscsi" in n:
+                            reboot_needed = True
+                    if reboot_needed:
+                        self.rdmc.ui.printer("Reboot is required for settings to take effect.\n")
             except NoChangesFoundOrMadeError as excp:
                 if returnvalue:
                     pass
@@ -212,10 +279,13 @@ class LoadCommand:
                                         self.rdmc.ui.error(str(err.sel))
                                 except:
                                     pass
-                raise redfish.ris.ValidationError(excp)
+                raise redfish.ris.ValidationError()
 
             if not results:
-                self.rdmc.ui.printer("No differences found from current configuration.\n")
+                if eth_conf:
+                    continue
+                else:
+                    self.rdmc.ui.printer("No differences found from current configuration.\n")
 
         # Return code
         if returnvalue:
@@ -234,7 +304,7 @@ class LoadCommand:
             options.latestschema = True
 
         try:
-            self.cmdbase.login_select_validation(self, options)
+            self.cmdbase.login_validation(self, options)
         except Exception:
             if options.mpfilename:
                 pass
@@ -263,9 +333,7 @@ class LoadCommand:
             tempholder = json.loads(filedata)
             return tempholder
         except:
-            raise InvalidFileFormattingError(
-                "Invalid file formatting found in file %s" % inputfile
-            )
+            raise InvalidFileFormattingError("Invalid file formatting found in file %s" % inputfile)
 
     def get_current_selector(self, path=None):
         """Returns current selected content minus hard coded list
@@ -322,10 +390,7 @@ class LoadCommand:
         os.mkdir(createdir)
 
         oofile = open(os.path.join(createdir, "CompleteOutputfile.txt"), "w+")
-        self.rdmc.ui.printer(
-            "Create multiple processes to load configuration "
-            "concurrently to all servers...\n"
-        )
+        self.rdmc.ui.printer("Create multiple processes to load configuration " "concurrently to all servers...\n")
 
         while True:
             if not self.queue.empty():
@@ -347,9 +412,7 @@ class LoadCommand:
 
             urlfilename = urlvar.split("//")[-1]
             logfile = open(os.path.join(createdir, urlfilename + ".txt"), "w+")
-            pinput = subprocess.Popen(
-                listargforsubprocess, shell=True, stdout=logfile, stderr=logfile
-            )
+            pinput = subprocess.Popen(listargforsubprocess, shell=True, stdout=logfile, stderr=logfile)
 
             processes.append((pinput, finput, urlfilename, logfile))
 
@@ -365,18 +428,12 @@ class LoadCommand:
             logfile.close()
 
             if returncode == 0:
-                self.rdmc.ui.printer(
-                    "Loading Configuration for {} : SUCCESS\n".format(urlfilename)
-                )
+                self.rdmc.ui.printer("Loading Configuration for {} : SUCCESS\n".format(urlfilename))
             else:
-                self.rdmc.ui.error(
-                    "Loading Configuration for {} : FAILED\n".format(urlfilename)
-                )
+                self.rdmc.ui.error("Loading Configuration for {} : FAILED\n".format(urlfilename))
                 self.rdmc.ui.error(
                     "ILOREST return code : {}.\nFor more details please check "
-                    "{}.txt under {} directory.\n".format(
-                        returncode, urlfilename, createdir
-                    )
+                    "{}.txt under {} directory.\n".format(returncode, urlfilename, createdir)
                 )
 
         oofile.close()
@@ -401,8 +458,7 @@ class LoadCommand:
 
         if not os.path.isfile(mpfile):
             raise InvalidFileInputError(
-                "File '%s' doesn't exist, please "
-                "create file by running save command." % mpfile
+                "File '%s' doesn't exist, please " "create file by running save command." % mpfile
             )
 
         try:
@@ -424,12 +480,8 @@ class LoadCommand:
                     args = shlex.split(line, posix=False)
 
                     if len(args) < 5:
-                        self.rdmc.ui.error(
-                            "Incomplete data in input file: {}\n".format(line)
-                        )
-                        raise InvalidMSCfileInputError(
-                            "Please verify the " "contents of the %s file" % mpfile
-                        )
+                        self.rdmc.ui.error("Incomplete data in input file: {}\n".format(line))
+                        raise InvalidMSCfileInputError("Please verify the " "contents of the %s file" % mpfile)
                     else:
                         linelist = globalargs + cmdtorun + args + cmdargs
                         line = str(line).replace("\n", "")
@@ -472,7 +524,6 @@ class LoadCommand:
             default=None,
         )
         customparser.add_argument(
-            "-o",
             "--outputdirectory",
             dest="outdirectory",
             help="""use the provided directory to output data for multiple server configuration""",
@@ -491,15 +542,13 @@ class LoadCommand:
             "--uniqueoverride",
             dest="uniqueoverride",
             action="store_true",
-            help="Override the measures stopping the tool from writing "
-            "over items that are system unique.",
+            help="Override the measures stopping the tool from writing " "over items that are system unique.",
             default=False,
         )
         customparser.add_argument(
             "--encryption",
             dest="encryption",
-            help="Optionally include this flag to encrypt/decrypt a file "
-            "using the key provided.",
+            help="Optionally include this flag to encrypt/decrypt a file " "using the key provided.",
             default=None,
         )
         customparser.add_argument(
@@ -510,3 +559,11 @@ class LoadCommand:
             " descriptions regarding the reboot flag, run help reboot.",
             default=None,
         )
+        customparser.add_argument(
+            "--force_network_config",
+            dest="force_network_config",
+            help="Use this flag to force set network configuration."
+            "Network settings will be skipped if the flag is not included.",
+            action="store_true",
+            default=None,
+        )
