@@ -19,18 +19,17 @@
 
 # ---------Imports---------
 from __future__ import unicode_literals
-from builtins import bytes, str, open, super, range, input, int, object
-import os
-import six
-import sys
 
+import os
+import sys
 from argparse import (
-    ArgumentParser,
-    _ArgumentGroup,
-    Action,
     SUPPRESS,
+    Action,
+    ArgumentParser,
     RawDescriptionHelpFormatter,
+    _ArgumentGroup,
 )
+from builtins import bytes, int, object, str, super
 
 from redfish.ris import NothingSelectedError
 
@@ -53,25 +52,11 @@ try:
 except ImportError:
     from ilorest.rdmc_helper import InvalidCommandLineErrorOPTS
 
+
 # from extensions.COMMANDS import LoginCommand, SelectCommand
 
 # ---------End of imports---------
 
-# Using hard coded list until better solution is found
-HARDCODEDLIST = [
-    "name",
-    "modified",
-    "type",
-    "description",
-    "attributeregistry",
-    "links",
-    "settingsresult",
-    "actions",
-    "availableactions",
-    "id",
-    "extref",
-]
-
 
 class _Verbosity(Action):
     def __init__(self, option_strings, dest, nargs, **kwargs):
@@ -129,20 +114,14 @@ class RdmcCommandBase(CommandBase):
     def __init__(self, name, usage, summary, aliases, argparser=None, **kwargs):
         """Constructor"""
         CommandBase.__init__(
-            self,
-            name=name,
-            usage=usage,
-            summary=summary,
-            aliases=aliases,
-            argparser=argparser,
-            **kwargs
+            self, name=name, usage=usage, summary=summary, aliases=aliases, argparser=argparser, **kwargs
         )
         self.json = False
         self.cache = False
         self.nologo = False
         self.toolbar = False
 
-    def login_select_validation(self, cmdinstance, options, skipbuild=False):
+    def login_validation(self, cmdinstance, options, skipbuild=False):
         """Combined validation function to login and select with other commands. Not for use with
         login or select commands themselves. Make sure your command imports options from
         add_login_arguments_group or there will be errors.
@@ -155,12 +134,7 @@ class RdmcCommandBase(CommandBase):
         :type skipbuild: bool.
         """
 
-        logobj = cmdinstance.rdmc.load_command(
-            cmdinstance.rdmc.search_commands("LoginCommand")
-        )
-        selobj = cmdinstance.rdmc.load_command(
-            cmdinstance.rdmc.search_commands("SelectCommand")
-        )
+        logobj = cmdinstance.rdmc.load_command(cmdinstance.rdmc.search_commands("LoginCommand"))
         inputline = list()
         client = None
         loggedin = False
@@ -176,17 +150,13 @@ class RdmcCommandBase(CommandBase):
                     inputline.extend([options.url])
                 if options.user:
                     if options.encode:
-                        options.user = rdmc_helper.Encryption.decode_credentials(
-                            options.user
-                        )
+                        options.user = rdmc_helper.Encryption.decode_credentials(options.user)
                         if isinstance(options.user, bytes):
                             options.user = options.user.decode("utf-8")
                     inputline.extend(["-u", options.user])
                 if options.password:
                     if options.encode:
-                        options.password = rdmc_helper.Encryption.decode_credentials(
-                            options.password
-                        )
+                        options.password = rdmc_helper.Encryption.decode_credentials(options.password)
                         if isinstance(options.password, bytes):
                             options.password = options.password.decode("utf-8")
                     inputline.extend(["-p", options.password])
@@ -220,21 +190,51 @@ class RdmcCommandBase(CommandBase):
             if options.path:
                 inputline.extend(["--path", options.path])
 
-        if getattr(options, "biospassword", False):
-            inputline.extend(["--biospassword", options.biospassword])
+            if getattr(options, "biospassword", False):
+                inputline.extend(["--biospassword", options.biospassword])
+            if getattr(options, "sessionid", False):
+                inputline.extend(["--sessionid", options.sessionid])
 
+            logobj.loginfunction(inputline, skipbuild=skipbuild)
+            loggedin = True
+
+        if not (loggedin or client or options.url or cmdinstance.rdmc.app.typepath.url):
+            message = "Local login initiated...\n"
+            if cmdinstance.rdmc.opts.verbose:
+                sys.stdout.write(message)
+            else:
+                rdmc_helper.LOGGER.info(message)
+
+        if not (loggedin or client):
+            logobj.loginfunction(inputline, skipbuild=skipbuild)
+
+    def login_select_validation(self, cmdinstance, options, skipbuild=False):
+        """Combined validation function to login and select with other commands. Not for use with
+        login or select commands themselves. Make sure your command imports options from
+        add_login_arguments_group or there will be errors.
+
+        :param cmdinstance: the command object instance
+        :type cmdinstance: list.
+        :param options: command line options
+        :type options: list.
+        :param skipbuild: flag to only login and skip monolith build
+        :type skipbuild: bool.
+        """
+        inputline = list()
+
+        self.login_validation(cmdinstance, options, skipbuild=skipbuild)
+        logobj = cmdinstance.rdmc.load_command(cmdinstance.rdmc.search_commands("LoginCommand"))
+        selobj = cmdinstance.rdmc.load_command(cmdinstance.rdmc.search_commands("SelectCommand"))
         if hasattr(options, "selector") and options.selector:
             if inputline:
                 inputline.extend(["--selector", options.selector])
                 logobj.loginfunction(inputline)
-                loggedin = True
             else:
                 if getattr(options, "ref", False):
                     inputline.extend(["--refresh"])
 
                 inputline.extend([options.selector])
                 selobj.selectfunction(inputline)
-                loggedin = True
         elif hasattr(options, "selector"):
             try:
                 inputline = list()
@@ -246,27 +246,10 @@ class RdmcCommandBase(CommandBase):
                 if selector:
                     inputline.extend([selector])
                     selobj.selectfunction(inputline)
-                loggedin = True
             except NothingSelectedError:
                 raise NothingSelectedError
-        if (
-            not loggedin
-            and not client
-            and not options.url
-            and not cmdinstance.rdmc.app.typepath.url
-        ):
-            try:
-                if cmdinstance.rdmc.opts.verbose:
-                    sys.stdout.write("Local login initiated...\n")
-                else:
-                    raise Exception
-            except Exception:
-                rdmc_helper.LOGGER.info("Local login initiated...\n")
-        if not loggedin and not client:
-            logobj.loginfunction(inputline, skipbuild=skipbuild)
 
     def logout_routine(self, cmdinstance, options):
-
         """Routine to logout of a server automatically at the completion of a command.
 
         :param commandinstance: the command object instance
@@ -275,9 +258,7 @@ class RdmcCommandBase(CommandBase):
         :type options: list.
         """
 
-        logoutobj = cmdinstance.rdmc.load_command(
-            cmdinstance.rdmc.search_commands("LogoutCommand")
-        )
+        logoutobj = cmdinstance.rdmc.load_command(cmdinstance.rdmc.search_commands("LogoutCommand"))
 
         if getattr(options, "logout", False):
             logoutobj.run("")
@@ -288,12 +269,8 @@ class RdmcCommandBase(CommandBase):
         :param parser: The parser to add the login option group to
         :type parser: ArgumentParser
         """
-        group = parser.add_argument_group(
-            "LOGIN OPTIONS", "Options for logging in to a system."
-        )
-        group.add_argument(
-            "--url", dest="url", help="Use the provided iLO URL to login.", default=None
-        )
+        group = parser.add_argument_group("LOGIN OPTIONS", "Options for logging in to a system.")
+        group.add_argument("--url", dest="url", help="Use the provided iLO URL to login.", default=None)
         group.add_argument(
             "--sessionid",
             dest="sessionid",
@@ -315,6 +292,13 @@ password and URL flags can be used to login to a server in the same command.""",
             help="""Use the provided iLO password to log in.""",
             default=None,
         )
+        group.add_argument(
+            "-o",
+            "--otp",
+            dest="login_otp",
+            help="""Use the provided iLO OTP to log in.""",
+            default=None,
+        )
         group.add_argument(
             "--biospassword",
             dest="biospassword",
@@ -416,8 +400,7 @@ class RdmcOptionParser(ArgumentParser):
 
     def __init__(self):
         super().__init__(
-            usage="%s [GLOBAL OPTIONS] [COMMAND] [COMMAND ARGUMENTS] "
-            "[COMMAND OPTIONS]" % versioning.__shortname__,
+            usage="%s [GLOBAL OPTIONS] [COMMAND] [COMMAND ARGUMENTS] " "[COMMAND OPTIONS]" % versioning.__shortname__,
             description="iLOrest is a command-line or interactive interface that allows users "
             "to manage Hewlett Packard Enterprise products that take advantage"
             " of RESTful APIs.\n\nIn order to view or manage a system you must"
@@ -443,9 +426,7 @@ class RdmcOptionParser(ArgumentParser):
             metavar="FILE",
         )
 
-        config_dir_default = os.path.join(
-            cliutils.get_user_config_dir(), ".%s" % versioning.__shortname__
-        )
+        config_dir_default = os.path.join(cliutils.get_user_config_dir(), ".%s" % versioning.__shortname__)
         self.add_argument(
             "--cache-dir",
             dest="config_dir",
@@ -463,9 +444,9 @@ class RdmcOptionParser(ArgumentParser):
             "Logging, Stdout, Stderr. '-vv': Level 2, Extends Level 1 with slightly "
             "elaborated iLO and HTTP response message. '-vvv': Level3, Extends Level 2 "
             "with message id, validation class, message text with embedded args, and "
-            "possible resolution/mitigation for iLO responses. High detailed HTTP responses. "
+            "possible resolution/mitigation for iLO responses. Includes HTTP responses. "
             "**NOTE 1**: Some responses may only contain limited information from the source."
-            "**NOTE 4**: Default level is 0.",
+            "**NOTE 2**: Default level is 0.",
             default=0,
         )
         self.add_argument(
@@ -483,6 +464,13 @@ class RdmcOptionParser(ArgumentParser):
             help="""Use the provided directory as the location for log file.""",
             metavar="PATH",
         )
+        self.add_argument(
+            "--nostdoutlog",
+            dest="nostdoutlog",
+            action="store_true",
+            help="""Disable debug logs to stdout.""",
+            default=False,
+        )
         self.add_argument(
             "--nocache",
             dest="nocache",
