@@ -20,38 +20,49 @@
 
 # ---------Imports---------
 from __future__ import unicode_literals
+
+import json
+import logging
 import os
 import sys
 import time
-import json
-import logging
-
 from collections import OrderedDict
-from ctypes import create_string_buffer, c_char_p, byref
+from ctypes import byref, c_char_p, create_string_buffer
 
-import six
 import pyaes
-
+import six
 from prompt_toolkit.completion import Completer, Completion
 
-import redfish.ris
 import redfish.hpilo.risblobstore2 as risblobstore2
+import redfish.ris
 
 try:
     import versioning
 except ImportError:
     from ilorest import versioning
 
-# from rdmc_base_classes import HARDCODEDLIST
-
 if os.name == "nt":
     from six.moves import winreg
-    from win32.lib.win32con import HKEY_LOCAL_MACHINE
+
+    if six.PY2:
+        from win32con import HKEY_LOCAL_MACHINE
+    elif six.PY3:
+        from win32.lib.win32con import HKEY_LOCAL_MACHINE
 
 # ---------End of imports---------
 
 
 # ---------Debug logger---------
+# Using hard coded list until better solution is found
+HARDCODEDLIST = [
+    "modified",
+    "attributeregistry",
+    "links",
+    "settingsresult",
+    "actions",
+    "availableactions",
+    "extref",
+]
 
 
 class InfoFilter(logging.Filter):
@@ -81,6 +92,7 @@ LOUT.addFilter(InfoFilter())
 # add logger handle
 LOGGER.addHandler(LOUT)
 
+
 # ---------End of debug logger---------
 
 
@@ -105,6 +117,7 @@ class ReturnCodes(object):
     UI_CLI_WARN_EXCEPTION = 11
     UI_CLI_USAGE_EXCEPTION = 12
     UI_CLI_COMMAND_NOT_FOUND_EXCEPTION = 13
+    INVALID_PASSWORD_LENGTH_ERROR = 14
 
     # ****** RMC/RIS ERRORS ******
     RIS_UNDEFINED_CLIENT_ERROR = 21
@@ -176,6 +189,7 @@ class ReturnCodes(object):
     DRIVE_MISSING_ERROR = 82
     PATH_UNAVAILABLE_ERROR = 83
     ILO_RIS_CORRUPTION_ERROR = 84
+    RESOURCE_NOT_READY_ERROR = 85
 
     # ****** RIS ERRORS ******
     RIS_RIS_BIOS_UNREGISTERED_ERROR = 100
@@ -185,13 +199,25 @@ class ReturnCodes(object):
     UPDATE_SERVICE_BUSY = 102
     FAILED_TO_UPLOAD_COMPONENT = 103
     TASKQUEUE_ERROR = 104
+    DEVICE_DISCOVERY_IN_PROGRESS = 105
+    INSTALLSET_ERROR = 106
+    INVALID_TARGET_ERROR = 107
 
-    # **** Compute Ops Manager Errors****
+    # **** ComputeOpsManagement Errors****
     CLOUD_CONNECT_TIMEOUT = 111
     CLOUD_CONNECT_FAILED = 112
     CLOUD_ALREADY_CONNECTED = 113
     PROXY_CONFIG_FAILED = 114
 
+    # **** scep error ****
+    SCEP_ENABLED_ERROR = 121
+
+    # *** authentication error ****
+    TFA_WRONG_OTP = 131
+    TFA_OTP_TIMEDOUT = 132
+    TFA_ENABLED_ERROR = 133
+    TFA_OTP_EMAILED = 134
+
     # ****** GENERAL ERRORS ******
     GENERAL_ERROR = 255
 
@@ -212,25 +238,31 @@ class ConfigurationFileError(RdmcError):
 
 
 class ProxyConfigFailedError(RdmcError):
-    """Raised when compute ops manager connection fails"""
+    """Raised when ComputeOpsManagement connection fails"""
 
     pass
 
 
 class CloudConnectTimeoutError(RdmcError):
-    """Raised when compute ops manager connection times out"""
+    """Raised when ComputeOpsManagement connection times out"""
 
     pass
 
 
 class CloudConnectFailedError(RdmcError):
-    """Raised when compute ops manager connection fails"""
+    """Raised when ComputeOpsManagement connection fails"""
+
+    pass
+
+
+class TfaEnablePreRequisiteError(RdmcError):
+    """Raised when pre-requisites not met while enabling TFA"""
 
     pass
 
 
 class AlreadyCloudConnectedError(RdmcError):
-    """Raised when compute ops manager is already connected"""
+    """Raised when ComputeOpsManagement is already connected"""
 
     pass
 
@@ -247,6 +279,12 @@ class iLORisCorruptionError(RdmcError):
     pass
 
 
+class ResourceNotReadyError(RdmcError):
+    """Raised when user tries to invoke a command that isn't enabled"""
+
+    pass
+
+
 class UsernamePasswordRequiredError(RdmcError):
     """Raised when username and password are required for local chif login"""
 
@@ -326,6 +364,12 @@ class InvalidOrNothingChangedSettingsError(RdmcError):
     pass
 
 
+class InvalidPasswordLengthError(RdmcError):
+    """Raised when password length is invalid"""
+
+    pass
+
+
 class NoDifferencesFoundError(RdmcError):
     """Raised when no differences are found in the current configuration"""
 
@@ -401,10 +445,6 @@ class IloLicenseError(RdmcError):
 
     pass
 
-class ScepenabledError(RdmcError):
-    """Raised when the generation csr or deletion of https cert is issues when scep is enabled"""
-
-    pass
 
 class ScepenabledError(RdmcError):
     """Raised when the generation csr or deletion of https cert is issues when scep is enabled"""
@@ -486,6 +526,12 @@ class TaskQueueError(RdmcError):
     pass
 
 
+class DeviceDiscoveryInProgress(RdmcError):
+    """Raised when device discovery in progress"""
+
+    pass
+
+
 class InvalidSmartArrayConfigurationError(RdmcError):
     """Raised for an invalid configuration of a smart array controller and/or disk configuration"""
 
@@ -498,6 +544,12 @@ class FallbackChifUse(RdmcError):
     pass
 
 
+class InstallsetError(RdmcError):
+    """Error while deleting Recovery installset"""
+
+    pass
+
+
 class UI(object):
     """UI class handles all of our printing etc so we have
     consistency across the project"""
@@ -515,29 +567,24 @@ class UI(object):
                     sys.stdout.write(str(data))
                     if flush:
                         sys.stdout.flush()
-                except IOError as excp:
+                except IOError:
                     pass
 
     def command_not_found(self, cmd):
         """Called when command was not found"""
         self.printer(
-            (
-                "\nCommand '%s' not found. Use the help command to "
-                "see a list of available commands\n" % cmd
-            ),
+            ("\nCommand '%s' not found. Use the help command to " "see a list of available commands\n" % cmd),
             excp=True,
         )
         return ReturnCodes.UI_CLI_COMMAND_NOT_FOUND_EXCEPTION
 
     def command_not_enabled(self, cmd, excp):
         """Called when command has not been enabled"""
-        self.printer(
-            ("\nCommand '%s' has not been enabled: %s\n" % (cmd, excp)), excp=excp
-        )
+        self.printer(("\nCommand '%s' has not been enabled: %s\n" % (cmd, excp)), excp=excp)
 
     def invalid_commmand_line(self, excp):
         """Called when user entered invalid command line entries"""
-        self.printer(("\nError: %s\n" % excp), excp=excp)
+        self.printer(("Error: %s\n" % excp), excp=excp)
 
     def ilo_ris_corruption(self, excp):
         """Called when user entered invalid command line entries"""
@@ -545,9 +592,7 @@ class UI(object):
 
     def standard_blob_error(self, excp):
         """Called when user error encountered with blob"""
-        self.printer(
-            ("\nError: Blob operation failed with error code %s\n" % excp), excp=excp
-        )
+        self.printer(("\nError: Blob operation failed with error code %s\n" % excp), excp=excp)
 
     def invalid_file_formatting(self, excp):
         """Called when file formatting is unrecognizable"""
@@ -609,8 +654,7 @@ class UI(object):
             self.printer(".", excp=True)
 
         self.printer(
-            "\nError: Could not authenticate. Invalid "
-            "credentials, or bad username/password.\n",
+            "\nError: Could not authenticate. Invalid " "credentials, or bad username/password.\n",
             excp=True,
         )
 
@@ -660,9 +704,7 @@ class UI(object):
         :type content: str.
         """
         # stringify
-        content = json.dumps(
-            content, indent=2, cls=redfish.ris.JSONEncoder, sort_keys=True
-        )
+        content = json.dumps(content, indent=2, cls=redfish.ris.JSONEncoder, sort_keys=True)
         self.printer(content, verbose_override=True)
         self.printer("\n")
 
@@ -788,9 +830,7 @@ class Encryption(object):
         except (UnicodeDecodeError, AttributeError):
             pass  # must be encoded already
         if Encryption.check_fips_mode_os():
-            raise CommandNotEnabledError(
-                "Encrypting of files is not available" " in FIPS mode."
-            )
+            raise CommandNotEnabledError("Encrypting of files is not available" " in FIPS mode.")
         if len(key) not in [16, 24, 32]:
             raise InvalidKeyError("")
         else:
@@ -822,8 +862,7 @@ class Encryption(object):
                 json.loads(decryptedfile)
             except:
                 raise UnableToDecodeError(
-                    "Unable to decrypt the file, make "
-                    "sure the key is the same as used in encryption."
+                    "Unable to decrypt the file, make " "sure the key is the same as used in encryption."
                 )
 
         return decryptedfile
@@ -882,7 +921,7 @@ class Encryption(object):
                 enc_val = retbuff.value.decode("utf-8")  # .encode('utf-8')
             if not retbuff.value:
                 raise UnableToDecodeError("")
-        except Exception as exp:
+        except Exception:
             raise UnableToDecodeError("Unable to decode credential %s." % credential)
 
         return enc_val
@@ -942,7 +981,7 @@ class TabAndHistoryCompletionClass(Completer):
                             if nested_info:
                                 if "properties" in nested_info:
                                     nested_info = nested_info["properties"]
-                                if not "AttributeName" in nested_info[token]:
+                                if "AttributeName" not in nested_info[token]:
                                     nested_info = (
                                         nested_info["properties"][token]
                                         if "properties" in nested_info
@@ -952,18 +991,13 @@ class TabAndHistoryCompletionClass(Completer):
                                     nested_info = nested_info[token]
                         except Exception:
                             break
-                    nested_data = (
-                        list(nested_data.keys()) if isinstance(nested_data, dict) else []
-                    )
+                    nested_data = list(nested_data.keys()) if isinstance(nested_data, dict) else []
                     lstoption = nested_data
 
                     # Try to get info for help bar
                     help_text = nested_info.get("HelpText", "")
                     enum_tab = []
-                    if (
-                        "Type" in nested_info
-                        and nested_info["Type"].lower() == "enumeration"
-                    ):
+                    if "Type" in nested_info and nested_info["Type"].lower() == "enumeration":
                         help_text += "\nPossible Values:\n"
                         for value in nested_info["Value"]:
                             enum_tab.append(value["ValueName"])
